@@ -1,6 +1,7 @@
 from pprint import pprint
 from collections import OrderedDict
 from packaging.version import Version
+from packaging.specifiers import SpecifierSet
 import os
 import shutil
 import requests
@@ -48,6 +49,7 @@ class Controller():
         except requests.exceptions.HTTPError as err:
             return resp.json()
         except requests.exceptions.RequestException as err:
+            self._session.offline = True
             self._session.log_error(err)
             return None
         else:
@@ -89,7 +91,7 @@ class Controller():
                 self.BASE_URL + '/token', data=data, allow_redirects=False
             )
             # Handle HTTPS redirect
-            if resp.status_code == 301:
+            if resp.status_code == 301 or resp.status_code == 308:
                 https_url = resp.headers['Location']
                 resp = requests.post(https_url, data=data)
                 if resp.status_code == 200:
@@ -99,6 +101,7 @@ class Controller():
         except requests.exceptions.HTTPError as err:
             return resp.json()
         except requests.exceptions.RequestException as err:
+            self._session.offline = True
             self._session.log_error(err)
             return None
         else:
@@ -299,7 +302,37 @@ class Controller():
 
         with open(f"{user_settings_folder}/user_servers_order.json", 'w+') as f:
             json.dump(dict(uid_list=data), f, indent=4)
+    
+    def fetch_project(self,name):
+        # Get projects data
+        data = self.api_get('/projects', json={})
+        
+        # Remove disable and archived projects
+        data = [
+            remote 
+            for remote in data
+            if remote['is_enabled']
+            and remote['is_archived'] is False
+        ]
+        # Filter projects to which user and site are assigned
+        data = [
+            remote
+            for remote in data
+            if self.check_project_has_user(remote['name'])
+            and self.check_project_has_site(remote['name'])
+        ]
 
+        #Get project by name
+        data = [
+            remote
+            for remote in data
+            if remote['name'] == name
+        ]
+        
+        return self._projects.fetch_projects(data, self.server_uid)[0]
+    
+    def fetch_cached_projects(self):
+        return self._projects.fetch_projects(None, self.server_uid)
 
     def fetch_projects(self):
         # Get projects data
@@ -332,6 +365,8 @@ class Controller():
         data = self.api_get(f'/projects/{uid}', json={})
         return self._projects.update_project(data, updated)
 
+    def get_server_from_project(self,name):
+        return self._projects.get_server_from_project(name)
     
     def update_project_env(self, project_name, project_uid):        
         # Fetch python path
@@ -438,92 +473,182 @@ class Controller():
             self._session.log_error('A major dependency is missing. Make sure you have kabaret, libreflow and a libreflow flow module in your project extensions.')
             return None, None
 
-        # Format for toml config
-        main_dependencies_string = """"""
-        for name, options in main_dependencies.items():
-            main_dependencies_string += f"{name} = "
-            if type(options) is not dict:
-                main_dependencies_string += f'"{options}"\n'
-            else:
-                for i, (option_name, option_value) in enumerate(main_dependencies[name].items()):
-                    if i > 0:
-                        if option_name == "allow-prereleases":
-                            main_dependencies_string += ', {name} = {value}'.format(name=option_name, value=option_value)
-                        else:
-                            main_dependencies_string += ', {name} = "{value}"'.format(name=option_name, value=option_value)
-                    else:
-                        main_dependencies_string += '{{ {name} = "{value}"'.format(name=option_name, value=option_value)
-                    
-                    if i == len(main_dependencies[name].items())-1:
-                        main_dependencies_string += ' }'
-                    
-                main_dependencies_string += "\n"
-        
-        extensions_dependencies_string = """"""
-        if len(extensions_dependencies) > 0:
-            for name, options in extensions_dependencies.items():
-                extensions_dependencies_string += f"{name} = "
-                if type(options) is not dict:
-                    extensions_dependencies_string += f'"{name}"\n'
-                else:
-                    for i, (option_name, option_value) in enumerate(extensions_dependencies[name].items()):
-                        if i > 0:
-                            extensions_dependencies_string += ', {name} = "{value}"'.format(name=option_name, value=option_value)
-                        else:
-                            extensions_dependencies_string += '{{ {name} = "{value}"'.format(name=option_name, value=option_value)
-                        
-                        if i == len(extensions_dependencies[name].items())-1:
-                            extensions_dependencies_string += ' }'
-                        
-                    extensions_dependencies_string += "\n"
-            
-        # Create toml content
-        toml_content = """
-            [tool.poetry]
-            name = "{project_name}-poetry"
-            version = "0.1.0"
-            description = ""
-            authors = ["Les Fees Speciales <voeu@les-fees-speciales.coop>"]
-
-            [tool.poetry.dependencies]
-            python = "{python_version}"
-            {main_list}
-            [tool.poetry.group.extensions.dependencies]
-            {extensions_list}
-            
-            [build-system]
-            requires = ["poetry-core"]
-            build-backend = "poetry.core.masonry.api"
-        """.format(
-            project_name=project_name,
-            python_version=self._settings.fetch_key('python_version'),
-            main_list=main_dependencies_string,
-            extensions_list=extensions_dependencies_string
-        )
-        
-        # Move current process to env folder
-        store_base_path = os.getcwd()
-        os.chdir(env_folder)
 
         update = True
         if not os.path.exists(toml_config_path):
             update = False
 
-        with open(toml_config_path, "w") as f:
-            f.write(toml_content)
-            f.close()
-
-        # If config exists, we use update command
-        if update:
-            subprocess.call([self.python_path, "-m", "poetry", "-n", "cache", "clear", "pypi", "--all"])
-            subprocess.call([self.python_path, "-m", "poetry", "env", "use", self.python_path])
-            subprocess.call([self.python_path, "-m", "poetry", "update"])
-        # Or we make the first install
         else:
-            subprocess.call([self.python_path, "-m", "poetry", "env", "use", self.python_path])
-            subprocess.call([self.python_path, "-m", "poetry", "install"])
+            data = toml.load(toml_config_path) 
+            if not data.get('project'):
+                self._session.log_info("Migrating poetry project to uv")
+                self.uninstall_project(project_uid)
+                update = False
+
         
-        os.chdir(store_base_path)
+        if not update :
+            python_version=self._settings.fetch_key('python_version')
+            # create toml
+            subprocess.call([
+                "uv","init", "--bare", 
+                "--name", f"{project_name}-uv",
+                "--author-from", "none", 
+                "--python", f"{python_version}", 
+                "--project",f"{env_folder}"
+            ])
+        
+        # else : subprocess.call(["uv","cache","clean"])
+        
+        # process is the same whether it is the init or not :
+
+        # add main dependecies
+        for name,options in main_dependencies.items() :
+
+            command = ["uv", "add"]
+
+            git = options.get('git')
+            if git:
+                git.replace(".git", "")
+                name += f" @ git+{options['git']}"
+
+            command.extend([f"{name}","--no-sync", "--project", f"{env_folder}"])
+
+            branch = options.get('branch')
+            if branch:
+                command.extend(["--branch", f"{branch}"])
+
+            prereleases = options.get('allow-prereleases')
+            if prereleases:
+                command.extend(["--prerelease", "allow"])
+            
+            subprocess.call(command)
+        
+        # add extensions dependecies
+        for name,options in extensions_dependencies.items() :
+
+            command = ["uv", "add"]
+
+            git = options.get('git')
+            if git:
+                git.replace(".git", "")
+                name += f" @ git+{options['git']}"
+
+            command.extend([f"{name}","--group", "extensions", "--no-sync", "--project", f"{env_folder}"])
+
+            branch = options.get('branch')
+            if branch:
+                command.extend(["--branch", f"{branch}"])
+
+            prereleases = options.get('allow-prereleases')
+            if prereleases:
+                command.extend(["--prerelease", "allow"])
+            
+            subprocess.call(command)
+        
+        # Remove extension dependencies no longer used :
+
+        toml_data = toml.load(toml_config_path)
+
+        if toml_data.get('dependency-groups'):
+            toml_list = toml_data.get('dependency-groups').get('extensions')
+            
+            for ext_string in toml_list:
+                ext_name = re.search(r"^[^=><~]*", ext_string).group()
+
+                if ext_name not in extensions_dependencies.keys():
+                    subprocess.call(["uv","remove", ext_name, "--no-sync", "--group", "extensions" ,"--project", f"{env_folder}"])
+
+        
+            
+
+        # # Format for toml config
+        # main_dependencies_string = """"""
+        # for name, options in main_dependencies.items():
+        #     main_dependencies_string += f"{name} = "
+        #     if type(options) is not dict:
+        #         main_dependencies_string += f'"{options}"\n'
+        #     else:
+        #         for i, (option_name, option_value) in enumerate(main_dependencies[name].items()):
+        #             if i > 0:
+        #                 if option_name == "allow-prereleases":
+        #                     main_dependencies_string += ', {name} = {value}'.format(name=option_name, value=option_value)
+        #                 else:
+        #                     main_dependencies_string += ', {name} = "{value}"'.format(name=option_name, value=option_value)
+        #             else:
+        #                 main_dependencies_string += '{{ {name} = "{value}"'.format(name=option_name, value=option_value)
+                    
+        #             if i == len(main_dependencies[name].items())-1:
+        #                 main_dependencies_string += ' }'
+                    
+        #         main_dependencies_string += "\n"
+        
+        # extensions_dependencies_string = """"""
+        # if len(extensions_dependencies) > 0:
+        #     for name, options in extensions_dependencies.items():
+        #         extensions_dependencies_string += f"{name} = "
+        #         if type(options) is not dict:
+        #             extensions_dependencies_string += f'"{name}"\n'
+        #         else:
+        #             for i, (option_name, option_value) in enumerate(extensions_dependencies[name].items()):
+        #                 if i > 0:
+        #                     extensions_dependencies_string += ', {name} = "{value}"'.format(name=option_name, value=option_value)
+        #                 else:
+        #                     extensions_dependencies_string += '{{ {name} = "{value}"'.format(name=option_name, value=option_value)
+                        
+        #                 if i == len(extensions_dependencies[name].items())-1:
+        #                     extensions_dependencies_string += ' }'
+                        
+        #             extensions_dependencies_string += "\n"
+            
+        # # Create toml content
+        # toml_content = """
+        #     [project]
+        #     name = "{project_name}-uv"
+        #     version = "0.1.0"
+        #     description = ""
+        #     authors = ["Les Fees Speciales <voeu@les-fees-speciales.coop>"]
+        #     requires-python = "{python_version}"
+
+        #     [tool.uv]
+        #     default-groups="all"
+
+        #     [dependency-groups]
+        #     main = [{main_list}]
+        #     extensions = [{extensions_list}]
+            
+        #     [build-system]
+        #     requires = ["uv_build>=0.7.15,<0.8"]
+        #     build-backend = "uv_build"
+        # """.format(
+        #     project_name=project_name,
+        #     python_version=self._settings.fetch_key('python_version'),
+        #     main_list=main_dependencies_string,
+        #     extensions_list=extensions_dependencies_string
+        # )
+        
+        # # Move current process to env folder
+        # store_base_path = os.getcwd()
+        # os.chdir(env_folder)
+
+        # update = True
+        # if not os.path.exists(toml_config_path):
+        #     update = False
+
+        # with open(toml_config_path, "w") as f:
+        #     f.write(toml_content)
+        #     f.close()
+
+        # # If config exists, we use update command
+        # if update:
+        #     subprocess.call([self.python_path, "-m", "poetry", "-n", "cache", "clear", "pypi", "--all"])
+        #     subprocess.call([self.python_path, "-m", "poetry", "env", "use", self.python_path])
+        #     subprocess.call([self.python_path, "-m", "poetry", "update"])
+        # # Or we make the first install
+        # else:
+        #     subprocess.call([self.python_path, "-m", "poetry", "env", "use", self.python_path])
+        #     subprocess.call([self.python_path, "-m", "poetry", "install"])
+        
+        # os.chdir(store_base_path)
 
         return data_resolve, env_folder
 
@@ -552,11 +677,8 @@ class Controller():
 
         # Format launch command with arguments
         cmd = [
-            f'"{self.python_path}"',
-            "-m",
-            "poetry",
+            "uv",
             "run",
-            "python",
             "-m",
             flow_module_name,
             "--host",
@@ -576,6 +698,10 @@ class Controller():
             "--search-index-uri",
             data_resolve["mongo_url"],
         ]
+
+        if extensions :
+            cmd.insert(cmd.index('-m'), "--group")
+            cmd.insert(cmd.index('-m'), "extensions")
         
         # Add redis replica arguments
         if data_resolve['env'].get("KABARET_READ_REPLICA_HOST"):
@@ -830,7 +956,7 @@ class Controller():
             if python_status is False:
                 self._session.log_error(f'Python path is invalid. Specify a correct version in the global settings.')
                 return
-            command = f'{self.python_path} -m poetry cache clear pypi --all'
+            # command = f'{self.python_path} -m uv cache clean'
 
         # Start with correct command
         if platform.system() == "Windows":
@@ -865,42 +991,66 @@ class Controller():
         store_base_path = os.getcwd()
         os.chdir(env_folder)
 
-        # If venv exists
-        env_info = subprocess.Popen(
-            [self.python_path, "-m", "poetry", "env", "info"],
-            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-            universal_newlines=True
-        )
-        env_status = re.search("(Valid).*(True)", env_info.communicate()[0])
+        toml_config_path = f"{env_folder}/pyproject.toml"
 
-        if env_status is None:
-            return None
-        else:
-            self._session.log_info(f'{project_name} env is valid.')
+        if not os.path.exists(toml_config_path):
+            return False
+
+        # If venv exists
+        # env_info = subprocess.Popen(
+        #     [self.python_path, "-m", "poetry", "env", "info"],
+        #     stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+        #     universal_newlines=True
+        # )
+        # env_status = re.search("(Valid).*(True)", env_info.communicate()[0])
+
+        # if env_status is None:
+        #     return None
+        # else:
+        #     self._session.log_info(f'{project_name} env is valid.')
         
         # Check correct python version
-        env_python_version = re.search(
-            f"(Python).*({self._settings.fetch_key('python_version')})", env_info.communicate()[0]
-        )
 
-        if env_python_version is None:
+        # with open(toml_config_path,"rb") as f:
+        data = toml.load(toml_config_path)
+
+        if not data.get('project'):
+            return False
+
+        vspec = SpecifierSet(data.get('project').get('requires-python'))
+
+        if self._settings.fetch_key('python_version') not in vspec:
             return False
         else:
             self._session.log_info(f'{project_name} python version is correct.')
 
-        # If all dependencies are installed
-        env_deps = subprocess.Popen(
-            [self.python_path, "-m", "poetry", "show"],
-            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-            universal_newlines=True
-        )
-        deps_warning = re.search("\(!\)", env_deps.communicate()[0])
+        # env_python_version = re.search(
+        #     f"(Python).*({self._settings.fetch_key('python_version')})", env_info.communicate()[0]
+        # )
 
-        # Return invalid status if no match or content in stderr
-        if deps_warning is not None or len(env_deps.communicate()[1]):
-            return False
-        else:
-            self._session.log_info(f'{project_name} dependencies are up to date.')
+        # if env_python_version is None:
+        #     return False
+        # else:
+        #     self._session.log_info(f'{project_name} python version is correct.')
+        
+
+        if not self._session.offline:
+
+            # If all dependencies are installed
+            env_deps = subprocess.Popen(
+                ["uv", "pip", "check"],
+                stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                universal_newlines=True
+            )
+
+            deps_warning = re.search("not installed", env_deps.communicate()[1])
+
+
+            # Return invalid status if no match or content in stderr
+            if deps_warning is not None:
+                return False
+            else:
+                self._session.log_info(f'{project_name} dependencies are up to date.')
 
         # Back current process to base path
         os.chdir(store_base_path)
@@ -932,13 +1082,17 @@ class Controller():
             stdout=subprocess.PIPE, stderr=subprocess.PIPE,
             universal_newlines=True
         )
-        env_name = re.search("([^\s]+)", env_info.communicate()[0]).group(0)
+        env_name = None 
+        r = re.search("([^\s]+)", env_info.communicate()[0])
+        if r:
+            env_name = r.group(0)
 
         # Remove poetry env
-        subprocess.call([self.python_path, "-m", "poetry", "env", "remove", env_name])
+        if env_name and env_name != ".venv":
+            subprocess.call([self.python_path, "-m", "poetry", "env", "remove", env_name])
 
         # Back current process to base path
-        os.chdir(store_base_path)
+        os.chdir(os.path.dirname(store_base_path))
 
         # Delete env folder
         shutil.rmtree(env_folder)
@@ -1006,23 +1160,16 @@ class Controller():
 
     def fetch_python_version(self, path):
         version_number = None
-        if platform.system() == "Windows":
-            try:
-                version_number = subprocess.check_output(f"{path} --version").decode()
-                version_number = re.findall(r'(?:(\d+\.(?:\d+\.)*\d+))', version_number)[0]
-            except IOError as e:
-                return None
-        elif platform.system() == "Linux":
-            try:
-                version_number = subprocess.Popen([path, '--version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                output_stdout = version_number.communicate()[0].decode()
-                output_stderr = version_number.communicate()[1].decode()
+        try:
+            version_number = subprocess.Popen([path, '--version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            output_stdout = version_number.communicate()[0].decode()
+            output_stderr = version_number.communicate()[1].decode()
 
-                version_number = re.findall(
-                    r'(?:(\d+\.(?:\d+\.)*\d+))', output_stdout if output_stderr == "" else output_stderr
-                )[0]
-            except IOError as e:
-                return None
+            version_number = re.findall(
+                r'(?:(\d+\.(?:\d+\.)*\d+))', output_stdout if output_stderr == "" else output_stderr
+            )[0]
+        except IOError as e:
+            return None
                 
         return version_number
 
