@@ -1,6 +1,5 @@
 import os
 import re
-import pprint
 import webbrowser
 import platform
 import subprocess
@@ -283,6 +282,10 @@ class MainWindow(QtWidgets.QWidget):
             self.overlay_enabled = True
 
         if redirect == 'projects':
+
+            if force_update:
+                self.projects.projects = []
+
             self.menu_bar.show()
             self.wizard.hide()
             self.settings_bar.hide()
@@ -316,12 +319,31 @@ class MainWindow(QtWidgets.QWidget):
                 else:
                     self.projects.show()
             else:
-                self.login.build()
-                self.login.show()
-                self.warning.hide()
-                self.projects.hide()
-                if self.menu_bar.servers.current_item:
-                    self.menu_bar.servers.current_item.refresh()
+                if self._session.offline:
+                    self.login.hide()
+                    self.setWindowTitle("Libreflow Launcher(Offline)")
+                    self.menu_bar.offline_icon.show()
+
+                    if init:
+                        self.notification_header.pop('Offline Mode', 'info', True)
+                    
+                    self.warning.hide()
+                    self.projects.refresh()
+                    if self.projects.count() == 0:
+                        self.projects.hide()
+                        self.warning.build('projects_empty')
+                        self.warning.show()
+                        return
+                    else:
+                        self.projects.show()
+
+                else:
+                    self.login.build()
+                    self.login.show()
+                    self.warning.hide()
+                    self.projects.hide()
+                    if self.menu_bar.servers.current_item:
+                        self.menu_bar.servers.current_item.refresh()
             
             return
         
@@ -880,6 +902,8 @@ class SettingsPage(QtWidgets.QFrame):
 
         self.page_lo = QtWidgets.QVBoxLayout(self)
 
+        self.settings_changed = False
+
         self.build()
 
     def build(self):
@@ -941,6 +965,9 @@ class SettingsPage(QtWidgets.QFrame):
         footer_lo.addWidget(self.save_btn)
     
     def refresh(self):
+
+        self.settings_changed = False
+
         self.current_item = self._sb.list.current_item
         if self.current_item.elided:
             self.page_text.setText(self.current_item.full_text)
@@ -994,7 +1021,7 @@ class SettingsPage(QtWidgets.QFrame):
         return self._ctrl.update_python_custom_paths(data)
 
     def _on_close_button_clicked(self):
-        self._mw.page_refresh('projects', True)
+        self._mw.page_refresh('projects', self.settings_changed)
 
     def _on_delete_button_clicked(self):
         if self._mw.popup.isVisible():
@@ -1003,6 +1030,7 @@ class SettingsPage(QtWidgets.QFrame):
             self._mw.menu_bar.servers.orderChanged.emit(self._mw.menu_bar.servers.get_order_data())
             self._sb.list.refresh(True)
             self._mw.settings_page.refresh()
+            self.settings_changed = True
             return self._mw.notification_header.pop('Server deleted', 'valid', True)
 
         self._mw.popup.pop('delete', 'server', self.page_text.text(), self._on_delete_button_clicked)
@@ -1048,6 +1076,8 @@ class SettingsPage(QtWidgets.QFrame):
         else:
             self._ctrl.update_settings(data)
         
+        self.settings_changed = True
+        
         self._mw.notification_header.pop("Saved", 'valid', True)
         self.toggle_footer()
 
@@ -1760,8 +1790,10 @@ class ServersList(QtWidgets.QScrollArea):
                 item = ServerItem(self, server, current_user)
                 self.layout.addWidget(item)
             
-            add_server = ServerAddButton(self)
-            self.layout.addWidget(add_server)
+
+            if not self._session.offline:
+                add_server = ServerAddButton(self)
+                self.layout.addWidget(add_server)
             
             # Left, top, right, bottom
             self.layout.setContentsMargins(0,0,0,0)
@@ -1841,7 +1873,17 @@ class MenuBar(QtWidgets.QFrame):
 
         self.servers = ServersList(self)
         vlo.addWidget(self.servers)
-        
+
+
+        self.offline_icon = QtWidgets.QPushButton(
+            resources.get_icon(('icons.gui', 'wifi_off')), ''
+        )
+        self.offline_icon.setObjectName('MenuBarButton')
+        self.offline_icon.setToolTip('The Overseer server is unavailable, Launcher is set to Offline mode')
+        vlo.addWidget(self.offline_icon, alignment=QtCore.Qt.AlignCenter)
+        self.offline_icon.hide()
+
+
         settings_btn = QtWidgets.QPushButton(
             resources.get_icon(('icons.gui', 'settings_outline'), color=QtGui.QColor('#26262f')), ''
         )
@@ -1853,6 +1895,7 @@ class MenuBar(QtWidgets.QFrame):
         self._mw.page_refresh('settings')
 
 
+
 class LoginPage(QtWidgets.QFrame):
 
     def __init__(self, mw):
@@ -1971,7 +2014,11 @@ class ProjectItem(QtWidgets.QWidget):
             QtGui.QIcon(resources.get_icon(('icons.gui', 'start'), color=QtGui.QColor('white'))), ''
         )
         self.start_button.setObjectName('ProjectButton')
-        self.start_button.clicked.connect(lambda: self._on_start_button_clicked())
+
+        if not self._session.offline:
+            self.start_button.clicked.connect(lambda: self._on_start_button_clicked())
+
+        else: self.start_button.clicked.connect(lambda: self._on_skip_updates_clicked())
 
         self.start_button.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
         self.start_button.customContextMenuRequested.connect(self._on_context_menu)
@@ -1984,7 +2031,11 @@ class ProjectItem(QtWidgets.QWidget):
             self._status = data['updated']
             self._data = data['data']
 
-        if self._status is True:
+        if self._session.offline :
+            color = QtGui.QColor('#DDD')
+            self.status_lbl.setToolTip("Offline - Update status unavailable")
+
+        elif self._status is True:
             color = QtGui.QColor('#4eb16f')
             self.status_lbl.setToolTip("Ready to launch")
         elif self._status is False:
@@ -2071,6 +2122,20 @@ class ProjectItem(QtWidgets.QWidget):
 
         self._mw.popup.hide()
 
+    def _on_skip_updates_clicked(self, exec_mode=None):
+
+        # Start the instance
+        self._session.log_info(f"Start {self._data['name']} libreflow {exec_mode if exec_mode else ''}")
+        start_status = self._ctrl.start_project(self._uid, self._data, exec_mode)
+        
+        # Update project status if error
+        if start_status is False:
+            self._mw.notification_header.pop("Error has occurred. Check console.", 'error', True)
+            updated_data = self._ctrl.update_project(self._uid, updated=None)
+            self.refresh(updated_data)
+
+        self._mw.popup.hide()
+
     def _on_uninstall_clicked(self):
         self._session.log_info(f"Uninstall {self._data['name']} libreflow")
         self._mw.popup.pop('uninstall')
@@ -2097,6 +2162,11 @@ class ProjectItem(QtWidgets.QWidget):
 
         context_menu.addAction(update_action)
 
+        skip_update_action = QtWidgets.QAction('Skip updates', self)
+        skip_update_action.triggered.connect(lambda: self._on_skip_updates_clicked())
+
+        context_menu.addAction(skip_update_action)
+
         if self._ctrl.fetch_advanced_mode():
             uninstall_action = QtWidgets.QAction('Uninstall', self)
             uninstall_action.triggered.connect(self._on_uninstall_clicked)
@@ -2149,6 +2219,7 @@ class ProjectsList(QtWidgets.QTableWidget):
         self._session = mw._session
 
         self.cellsContent = []
+        self.projects = []
 
         ### For grid view
         # self.cellWidth = 335
@@ -2187,10 +2258,13 @@ class ProjectsList(QtWidgets.QTableWidget):
             
             self.cellsContent = []
 
-        projects = self._ctrl.fetch_projects()
-        projects = sorted(projects, key=lambda p: p['data']['name'])
+        if self.projects == [] :
+            if not self._session.offline:
+                self.projects = self._ctrl.fetch_projects()
+            else : self.projects = self._ctrl.fetch_cached_projects()
+        _projects = sorted(self.projects, key=lambda p: p['data']['name'])
 
-        for project in projects:
+        for project in _projects:
             start_r, start_c = self.get_last_idx()
             if start_c == 0:
                 self.setRowCount(start_r + 1)
